(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
    typeof define === 'function' && define.amd ? define(['exports'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global["cstffx-search"] = {}));
})(this, (function (exports) { 'use strict';

    exports.TextSearchPosition = void 0;
    (function (TextSearchPosition) {
        TextSearchPosition[TextSearchPosition["Start"] = 0] = "Start";
        TextSearchPosition[TextSearchPosition["Any"] = 1] = "Any";
        TextSearchPosition[TextSearchPosition["End"] = 2] = "End";
    })(exports.TextSearchPosition || (exports.TextSearchPosition = {}));

    /**
     * Simple regular expresion builder
     */
    class TextSearchRegex {
        /**
         * Build a regular expression according to the options
         * @param query
         * @param options
         */
        static build(query, options) {
            const flags = [];
            let pattern = "";
            if (options?.ignoreCase) {
                flags.push("i");
            }
            for (let char of query) {
                let variant = options?.alias ? options?.alias.get(char) : false;
                char = TextSearchRegex.scape(char);
                if (variant) {
                    variant = TextSearchRegex.scape(variant);
                    pattern += `(${char}|${variant})`;
                }
                else {
                    pattern += char;
                }
            }
            if (options?.word) {
                pattern = `\\b${pattern}\\b`;
            }
            switch (options?.position) {
                case exports.TextSearchPosition.Start:
                    pattern = `^${pattern}`;
                    break;
                case exports.TextSearchPosition.End:
                    pattern = `${pattern}\$`;
                    break;
            }
            return new RegExp(pattern, flags.join(""));
        }
        /**
         * Escape special characters
         * @param char
         * @private
         */
        static scape(char) {
            // Thanks to https://stackoverflow.com/questions/3446170/escape-string-for-use-in-javascript-regex
            return char.replace(/[.*+?^${}()|[\]\\]/g, "\\$&");
        }
    }

    /**
     * Allow to reuse a regular expresion with different strings input
     */
    class TextSearch {
        regex;
        lastQuery;
        options;
        /**
         *
         * @param query
         * @param options
         */
        constructor(query, options) {
            this.options = options || {};
            if (query) {
                this.updateRegex(query);
            }
        }
        /**
         * Return true if str match the regular expresion
         * @param input
         */
        match(input, query) {
            if (query && query !== this.lastQuery) {
                this.updateRegex(query);
            }
            return this.regex ? !!input.match(this.regex) : false;
        }
        /**
         * Set the current options
         * @param options
         */
        setOptions(options) {
            this.options = options;
            if (this.lastQuery) {
                this.updateRegex(this.lastQuery);
            }
        }
        /**
         * Update the current regex
         * @param query
         * @param options
         * @private
         */
        updateRegex(query) {
            this.regex = TextSearchRegex.build(query, this.options);
        }
    }

    exports.TextSearch = TextSearch;
    exports.TextSearchRegex = TextSearchRegex;

    Object.defineProperty(exports, '__esModule', { value: true });

}));
